/*
*
* $Id: util.c,v 1.3 2001/06/02 07:10:45 konst Exp $
*
* Copyright (C) 2000 Denis V. Dmitrienko <denis@null.net>
* Copyright (C) 2000 Bill Soudan <soudan@kde.org>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*
*/

#include <stdlib.h>
#include <ctype.h>

#ifndef _WIN32
#include <unistd.h>
#endif

#include "icqtypes.h"
#include "icq.h"
#include "icqlib.h"
#include "stdpackets.h"
#include "util.h"
#include "stdarg.h"

icq_ArrayType icq_Countries[] = {
  { "Afghanistan", 93 },
  { "Albania", 355 },
  { "Algeria", 213 },
  { "American Samoa", 684 },
  { "Andorra", 376 },
  { "Angola", 244 },
  { "Anguilla", 101 },
  { "Antigua", 102 },
  { "Argentina", 54 },
  { "Armenia", 374 },
  { "Aruba", 297 },
  { "Ascension Island", 247 },
  { "Australia", 61 },
  { "Australian Antarctic Territory", 6721 },
  { "Austria", 43 },
  { "Azerbaijan", 994 },
  { "Bahamas", 103 },
  { "Bahrain", 973 },
  { "Bangladesh", 880 },
  { "Barbados", 104 },
  { "Barbuda", 120 },
  { "Belarus", 375 },
  { "Belgium", 32 },
  { "Belize", 501 },
  { "Benin", 229 },
  { "Bermuda", 105 },
  { "Bhutan", 975 },
  { "Bolivia", 591 },
  { "Bosnia and Herzegovina", 387 },
  { "Botswana", 267 },
  { "Brazil", 55 },
  { "British Virgin Islands", 106 },
  { "Brunei", 673 },
  { "Bulgaria", 359 },
  { "Burkina Faso", 226 },
  { "Burundi", 257 },
  { "Cambodia", 855 },
  { "Cameroon", 237 },
  { "Canada", 107 },
  { "Cape Verde Islands", 238 },
  { "Cayman Islands", 108 },
  { "Central African Republic", 236 },
  { "Chad", 235 },
  { "Chile", 56 },
  { "China", 86 },
  { "Christmas Island", 672 },
  { "Cocos-Keeling Islands", 6101 },
  { "Colombia", 57 },
  { "Comoros", 2691 },
  { "Congo", 242 },
  { "Cook Islands", 682 },
  { "Costa Rica", 506 },
  { "Croatia", 385 },
  { "Cuba", 53 },
  { "Cyprus", 357 },
  { "Czech Republic", 42 },
  { "Denmark", 45 },
  { "Diego Garcia", 246 },
  { "Djibouti", 253 },
  { "Dominica", 109 },
  { "Dominican Republic", 110 },
  { "Ecuador", 593 },
  { "Egypt", 20 },
  { "El Salvador", 503 },
  { "Equatorial Guinea", 240 },
  { "Eritrea", 291 },
  { "Estonia", 372 },
  { "Ethiopia", 251 },
  { "Faeroe Islands", 298 },
  { "Falkland Islands", 500 },
  { "Fiji Islands", 679 },
  { "Finland", 358 },
  { "France", 33 },
  { "French Antilles", 5901 },
  { "French Guiana", 594 },
  { "French Polynesia", 689 },
  { "Gabon", 241 },
  { "Gambia", 220 },
  { "Georgia", 995 },
  { "Germany", 49 },
  { "Ghana", 233 },
  { "Gibraltar", 350 },
  { "Greece", 30 },
  { "Greenland", 299 },
  { "Grenada", 111 },
  { "Guadeloupe", 590 },
  { "Guam", 671 },
  { "Guantanamo Bay", 5399 },
  { "Guatemala", 502 },
  { "Guinea", 224 },
  { "Guinea-Bissau", 245 },
  { "Guyana", 592 },
  { "Haiti", 509 },
  { "Honduras", 504 },
  { "Hong Kong", 852 },
  { "Hungary", 36 },
  { "Iceland", 354 },
  { "India", 91 },
  { "Indonesia", 62 },
  { "Iran", 98 },
  { "Iraq", 964 },
  { "Ireland", 353 },
  { "Israel", 972 },
  { "Italy", 39 },
  { "Ivory Coast", 225 },
  { "Jamaica", 112 },
  { "Japan", 81 },
  { "Jordan", 962 },
  { "Kazakhstan", 705 },
  { "Kenya", 254 },
  { "Kiribati Republic", 686 },
  { "Korea (North)", 850 },
  { "Korea (Republic of)", 82 },
  { "Kuwait", 965 },
  { "Kyrgyz Republic", 706 },
  { "Laos", 856 },
  { "Latvia", 371 },
  { "Lebanon", 961 },
  { "Lesotho", 266 },
  { "Liberia", 231 },
  { "Libya", 218 },
  { "Liechtenstein", 4101 },
  { "Lithuania", 370 },
  { "Luxembourg", 352 },
  { "Macau", 853 },
  { "Madagascar", 261 },
  { "Malawi", 265 },
  { "Malaysia", 60 },
  { "Maldives", 960 },
  { "Mali", 223 },
  { "Malta", 356 },
  { "Marshall Islands", 692 },
  { "Martinique", 596 },
  { "Mauritania", 222 },
  { "Mauritius", 230 },
  { "Mayotte Island", 269 },
  { "Mexico", 52 },
  { "Micronesia, Federated States of", 691 },
  { "Moldova", 373 },
  { "Monaco", 377 },
  { "Mongolia", 976 },
  { "Montserrat", 113 },
  { "Morocco", 212 },
  { "Mozambique", 258 },
  { "Myanmar", 95 },
  { "Namibia", 264 },
  { "Nauru", 674 },
  { "Nepal", 977 },
  { "Netherlands Antilles", 599 },
  { "Netherlands", 31 },
  { "Nevis", 114 },
  { "New Caledonia", 687 },
  { "New Zealand", 64 },
  { "Nicaragua", 505 },
  { "Niger", 227 },
  { "Nigeria", 234 },
  { "Niue", 683 },
  { "Norfolk Island", 6722 },
  { "Norway", 47 },
  { "Oman", 968 },
  { "Pakistan", 92 },
  { "Palau", 680 },
  { "Panama", 507 },
  { "Papua New Guinea", 675 },
  { "Paraguay", 595 },
  { "Peru", 51 },
  { "Philippines", 63 },
  { "Poland", 48 },
  { "Portugal", 351 },
  { "Puerto Rico", 121 },
  { "Qatar", 974 },
  { "Republic of Macedonia", 389 },
  { "Reunion Island", 262 },
  { "Romania", 40 },
  { "Rota Island", 6701 },
  { "Russia", 7 },
  { "Rwanda", 250 },
  { "Saint Lucia", 122 },
  { "Saipan Island", 670 },
  { "San Marino", 378 },
  { "Sao Tome and Principe", 239 },
  { "Saudi Arabia", 966 },
  { "Senegal Republic", 221 },
  { "Seychelle Islands", 248 },
  { "Sierra Leone", 232 },
  { "Singapore", 65 },
  { "Slovak Republic", 4201 },
  { "Slovenia", 386 },
  { "Solomon Islands", 677 },
  { "Somalia", 252 },
  { "South Africa", 27 },
  { "Spain", 34 },
  { "Sri Lanka", 94 },
  { "St. Helena", 290 },
  { "St. Kitts", 115 },
  { "St. Pierre and Miquelon", 508 },
  { "St. Vincent and the Grenadines", 116 },
  { "Sudan", 249 },
  { "Suriname", 597 },
  { "Swaziland", 268 },
  { "Sweden", 46 },
  { "Switzerland", 41 },
  { "Syria", 963 },
  { "Taiwan, Republic of China", 886 },
  { "Tajikistan", 708 },
  { "Tanzania", 255 },
  { "Thailand", 66 },
  { "Tinian Island", 6702 },
  { "Togo", 228 },
  { "Tokelau", 690 },
  { "Tonga", 676 },
  { "Trinidad and Tobago", 117 },
  { "Tunisia", 216 },
  { "Turkey", 90 },
  { "Turkmenistan", 709 },
  { "Turks and Caicos Islands", 118 },
  { "Tuvalu", 688 },
  { "USA", 1 },
  { "Uganda", 256 },
  { "Ukraine", 380 },
  { "United Arab Emirates", 971 },
  { "United Kingdom", 44 },
  { "United States Virgin Islands", 123 },
  { "Uruguay", 598 },
  { "Uzbekistan", 711 },
  { "Vanuatu", 678 },
  { "Vatican City", 379 },
  { "Venezuela", 58 },
  { "Vietnam", 84 },
  { "Wallis and Futuna Islands", 681 },
  { "Western Samoa", 685 },
  { "Yemen", 967 },
  { "Yugoslavia", 381 },
  { "Zaire", 243 },
  { "Zambia", 260 },
  { "Zimbabwe", 263 },
  { "Not entered", 0xffff }
};

icq_ArrayType icq_MetaOccupation[] = {
  {"Academic", 1},
  {"Administrative", 2},
  {"Art/Entertainment", 3},
  {"College Student", 4},
  {"Computers", 5},
  {"Community & Social", 6},
  {"Education", 7},
  {"Engineering", 8},
  {"Financial Services", 9},
  {"Government", 10},
  {"High School Student", 11},
  {"Home", 12},
  {"ICQ - Providing Help", 13},
  {"Law", 14},
  {"Managerial", 15},
  {"Manufacturing", 16},
  {"Medical/Health", 17},
  {"Military", 18},
  {"Non-Government Organization", 19},
  {"Professional", 20},
  {"Retail", 21},
  {"Retired", 22},
  {"Science & Research", 23},
  {"Sports", 24},
  {"Technical", 25},
  {"University Student", 26},
  {"Web Building", 27},
  {"Other Services", 99},
};

icq_ArrayType icq_MetaPastBackgrounds[] = {
  {"Elementary School", 300},
  {"High School", 301},
  {"College", 302},
  {"University", 303},
  {"Military", 304},
  {"Past Work Place", 305},
  {"Past Organization", 306},
  {"Other", 399},
};

icq_ArrayType icq_MetaAffiliations[] = {
  {"Alumni Org.", 200},
  {"Charity Org.", 201},
  {"Club/Social Org.", 202},
  {"Community Org.", 203},
  {"Cultural Org.", 204},
  {"Fan Clubs", 205},
  {"Fraternity/Sorority", 206},
  {"Hobbyists Org.", 207},
  {"International Org.", 208},
  {"Nature and Environment Org.", 209},
  {"Professional Org.", 210},
  {"Scientific/Technical Org.", 211},
  {"Self Improvement Group", 212},
  {"Spiritual/Religious Org.", 213},
  {"Sports Org.", 214},
  {"Support Org.", 215},
  {"Trade and Business Org.", 216},
  {"Union", 217},
  {"Voluntary Org.", 218},
  {"Other", 299},
};

icq_ArrayType icq_MetaLanguages[] = {
  {"Not Entered", 0},
  {"Arabic", 1},
  {"Bhojpuri", 2},
  {"Bulgarian", 3},
  {"Burmese", 4},
  {"Cantonese", 5},
  {"Catalan", 6},
  {"Chinese", 7},
  {"Croatian", 8},
  {"Czech", 9},
  {"Danish", 10},
  {"Dutch", 11},
  {"English", 12},
  {"Esperanto", 13},
  {"Estonian", 14},
  {"Farsi", 15},
  {"Finnish", 16},
  {"French", 17},
  {"Gaelic", 18},
  {"German", 19},
  {"Greek", 20},
  {"Hebrew", 21},
  {"Hindi", 22},
  {"Hungarian", 23},
  {"Icelandic", 24},
  {"Indonesian", 25},
  {"Italian", 26},
  {"Japanese", 27},
  {"Khmer", 28},
  {"Korean", 29},
  {"Lao", 30},
  {"Latvian", 31},
  {"Lithuanian", 32},
  {"Malay", 33},
  {"Norwegian", 34},
  {"Polish", 35},
  {"Portuguese", 36},
  {"Romanian", 37},
  {"Russian", 38},
  {"Serbian", 39},
  {"Slovak", 40},
  {"Slovenian", 41},
  {"Somali", 42},
  {"Spanish", 43},
  {"Swahili", 44},
  {"Swedish", 45},
  {"Tagalog", 46},
  {"Tatar", 47},
  {"Thai", 48},
  {"Turkish", 49},
  {"Ukrainian", 50},
  {"Urdu", 51},
  {"Vietnamese", 52},
  {"Yiddish", 53},
  {"Yoruba", 54},
  {"Afrikaans", 55},
  {"Bosnian", 56},
  {"Persian", 57},
  {"Albanian", 58},
  {"Armenian", 59},
  {"Punjabi", 60},
  {"Chamorro", 61},
  {"Mongolian", 62},
  {"Mandarin", 63},
  {"Taiwaness", 64},
  {"Macedonian", 65},
  {"Sindhi", 66},
  {"Welsh", 67},
};

void hex_dump(char *data, long size)
{
  long i;
  long x;
  int r=1;
  char d[32];
  unsigned char bfr[64];
  for(i=0, x=0;;i++)
  {
    if(i<size)
    {
      if(x==0)
	printf("%04lx: ", i);
#ifdef _WIN32
      sprintf(d, "%08x", data[i]);
#else
      snprintf(d, 9, "%08x", data[i]);
#endif
      printf("%c%c ", d[6], d[7]);
      bfr[x]=data[i];
      if(bfr[x]<0x20)
	bfr[x]='.';
      if(bfr[x]>0x7f)
	bfr[x]='.';
    }
    else
    {
      if(x==0)
	break;
      else
      {
	printf("   ");
	bfr[x]=' ';
	r=0;
      }
    }
    x++;
    if(!(x<16))
    {
      bfr[x]=0;
      printf("%s\n", bfr);
      x=0;
      if(!r)
	break;
    }
  }
}

/* simple mapping for udp status->tcp status translation function */
struct icq_TCPStatusRec
{
    unsigned long udp_status;
    long tcp_status;
} icq_TCPStatusMap[] = {
    { STATUS_OFFLINE     , 0                        }, 
    { STATUS_ONLINE      , ICQ_TCP_STATUS_ONLINE    }, 
    { STATUS_INVISIBLE   , ICQ_TCP_STATUS_INVISIBLE },
    { STATUS_NA          , ICQ_TCP_STATUS_NA        },
    { STATUS_FREE_CHAT   , ICQ_TCP_STATUS_FREE_CHAT },
    { STATUS_OCCUPIED    , ICQ_TCP_STATUS_OCCUPIED  },
    { STATUS_AWAY        , ICQ_TCP_STATUS_AWAY      },
    { STATUS_DND         , ICQ_TCP_STATUS_DND       },
};

long icq_TCPXlateStatus(unsigned long udp_status)
{
  unsigned int i;
  for(i=0;i<sizeof(icq_TCPStatusMap)/sizeof(struct icq_TCPStatusRec);i++)
    if(icq_TCPStatusMap[i].udp_status==udp_status) 
      return icq_TCPStatusMap[i].tcp_status;
  /* warning: "couldn't find tcp status for %d, assuming 0", udp_status */
  return 0;
}

void icq_FmtLog(ICQLINK *link, int level, const char *fmt, ...)
{
  char buffer[2048];
  va_list ap;
  va_start(ap, fmt);
  
  if(!link)
    return;
#ifdef _MSVC_
  _vsnprintf(buffer, 1024, fmt, ap);
#else
  (void)vsnprintf(buffer, 1024, fmt, ap);
#endif
  va_end(ap);

  if(link->icq_Log && icq_LogLevel>=level)
    (*link->icq_Log)(link, time(0L), level, buffer);
}

/**
 * Following functions used by qsort and bsearch to
 * compare 2 pointers to icq_ArrayType object.
 */
int array_code_compare(const void *x, const void *y)
{
  return ((icq_ArrayType*)x)->code - ((icq_ArrayType*)y)->code;
}

const char *icq_GetCountryName(unsigned short code)
{
  int i;

  for(i = 0; ; i++)
  {
    if(icq_Countries[i].code == code) return icq_Countries[i].name;
    else if(icq_Countries[i].code == 0xffff) break;
  }

  return "Unknown";
}

const char *icq_GetMetaOccupationName(unsigned short code)
{
  icq_ArrayType *res,key;
    
  key.code=code;
  res=bsearch(&key, icq_MetaOccupation, sizeof(icq_MetaOccupation)/sizeof(icq_ArrayType),
	      sizeof(icq_ArrayType), array_code_compare);
    
  return res?res->name:"Unknown";
}

const char *icq_GetMetaBackgroundName(unsigned short code)
{
  icq_ArrayType *res,key;
    
  key.code=code;
  res=bsearch(&key, icq_MetaPastBackgrounds, sizeof(icq_MetaPastBackgrounds)/sizeof(icq_ArrayType),
	      sizeof(icq_ArrayType), array_code_compare);
    
  return res?res->name:"Unknown";
}

const char *icq_GetMetaAffiliationName(unsigned short code)
{
  icq_ArrayType *res,key;
    
  key.code=code;
  res=bsearch(&key, icq_MetaAffiliations, sizeof(icq_MetaAffiliations)/sizeof(icq_ArrayType),
	      sizeof(icq_ArrayType), array_code_compare);
    
  return res?res->name:"Unknown";
}

const char *icq_GetMetaLanguageName(unsigned short code)
{
  icq_ArrayType *res,key;
    
  key.code=code;
  res=bsearch(&key, icq_MetaLanguages, sizeof(icq_MetaLanguages)/sizeof(icq_ArrayType),
	      sizeof(icq_ArrayType), array_code_compare);
    
  return res?res->name:"Unknown";
}

/********************************************
returns a string describing the status or
a "Error" if no such string exists
*********************************************/
const char *icq_ConvertStatus2Str(unsigned long status)
{
  if((unsigned long)STATUS_OFFLINE == status) /* this because -1 & 0x01FF is not -1 */
    return "Offline";

  if((status & STATUS_INVISIBLE) == STATUS_INVISIBLE)
    return "Invisible";
  else if((status & STATUS_FREE_CHAT) == STATUS_FREE_CHAT)
    return "Free for chat";
  else if((status & STATUS_DND) == STATUS_DND)
    return "Do not disturb";
  else if((status & STATUS_OCCUPIED) == STATUS_OCCUPIED)
    return "Occupied";
  else if((status & STATUS_NA) == STATUS_NA)
    return "Not available";
  else if((status & STATUS_AWAY) == STATUS_AWAY)
    return "Away";
  else if(!(status & 0x01FF))
    return "Online";
  else
    return "Error";
}
